@@ -21,6 +21,7 @@ Rez builder help to build rez packages.
 
 # Import built-in modules
 import abc
+import logging
 import os
 import platform
 import re
@@ -32,9 +33,13 @@ import tempfile
 import zipfile
 
 # Import local modules
+from rezbuild.bin_utils import make_bin_movable
 from rezbuild.bin_utils import make_bins_movable
-from rezbuild.constant import SHELL_CONTENT
+from rezbuild.constants import SHELL_CONTENT
 from rezbuild.exceptions import ArgumentError
+from rezbuild.exceptions import InstallerNotFoundError
+from rezbuild.exceptions import NotFoundPythonInBinError
+from rezbuild.exceptions import ReNotMatchError
 from rezbuild.exceptions import UnsupportedError
 from rezbuild.utils import clear_path
 from rezbuild.utils import copy_tree
@@ -54,9 +59,6 @@ class RezBuilder(abc.ABC):
         self.source_path = os.environ["REZ_BUILD_SOURCE_PATH"]
         self.variant_index = os.environ["REZ_BUILD_VARIANT_INDEX"]
         self.workspace = os.path.join(self.build_path, "workspace")
-        # self.temp_dirs list all the temporary directory objects that create
-        # by tempfile.
-        self.temp_dirs = []
         super().__init__(**kwargs)
 
     def build(self, **kwargs):
@@ -64,12 +66,6 @@ class RezBuilder(abc.ABC):
         self.create_work_dir()
         self.custom_build(**kwargs)
         self.install()
-        self.clean_temp_dir()
-
-    def clean_temp_dir(self):
-        """Clean all the temp dir."""
-        for temp_dir in self.temp_dirs:
-            temp_dir.cleanup()
 
     def create_work_dir(self):
         """Create the work directory.
@@ -446,6 +442,23 @@ class MacOSDmgBuilder(MacOSBuilder, InstallBuilder):
 class PythonBuilder(RezBuilder, abc.ABC):
     """This class include some common method for build python package."""
 
+    @staticmethod
+    def determine_python(filepath):
+        """Determine the entry point is using python.exe or pythonw.exe.
+
+        Args:
+            filepath (str): Path of the bin entry points.
+        """
+        with open(filepath, "rb") as file:
+            content = file.read()
+        if b"python.exe" in content:
+            return "python.exe"
+        elif b"pythonw.exe" in content:
+            return "pythonw.exe"
+        else:
+            raise NotFoundPythonInBinError(
+                f"Not found python executable path in {filepath}")
+
     def change_shebang(self, root="", shebang=""):
         """Change all the shebang of entry files.
 
@@ -455,12 +468,23 @@ class PythonBuilder(RezBuilder, abc.ABC):
             shebang (str): The shebang content you want to change to.
         """
         root = root or os.path.join(self.workspace, "bin")
-        if platform.system() == "Windows":
-            shebang = shebang or "#!python.exe"
-            make_bins_movable(root, shebang, "#!.+python.exe")
-        else:
-            shebang = shebang or "/usr/bin/env python"
-            make_bins_movable(root, shebang)
+        for filename in os.listdir(root):
+            bin_file = os.path.join(root, filename)
+            if platform.system() == "Windows":
+                try:
+                    shebang_ = (
+                        shebang or "#!" + self.determine_python(bin_file))
+                    # https://regex101.com/r/HENaAh/1
+                    make_bin_movable(bin_file, shebang_, "#!.+pythonw?.exe")
+                except NotFoundPythonInBinError as e:
+                    logging.getLogger(__name__).warning(str(e))
+                except ReNotMatchError:
+                    logging.getLogger(__name__).warning(
+                        "Shebang regex #!.+pythonw?.exe not match, skip "
+                        "changing shebang")
+            else:
+                shebang = shebang or "/usr/bin/env python"
+                make_bin_movable(bin_file, shebang)
 
     @staticmethod
     def install_wheel_by_pip(wheel_file, install_path):
@@ -481,7 +505,7 @@ class PythonBuilder(RezBuilder, abc.ABC):
             shebang=""):
         """Install wheel file.
 
-        This function will install wheel file, move python pakcages into
+        This function will install wheel file, move python packages into
         site-packages directory and change shebang if needed.
 
         Args:
@@ -494,7 +518,6 @@ class PythonBuilder(RezBuilder, abc.ABC):
         """
         install_path = install_path or self.workspace
         self.install_wheel_by_pip(wheel_file, install_path)
-        self.to_site_packages()
         if change_shebang:
             self.change_shebang(shebang=shebang)
 
@@ -544,15 +567,21 @@ class PythonSourceBuilder(PythonBuilder):
             temp_src = os.path.join(temp_dir, "src")
             shutil.copytree(source_root, temp_src)
             wheel_dir = os.path.join(self.build_path, "wheel_dir")
+            if os.path.exists(wheel_dir):
+                remove_tree(wheel_dir)
+            os.makedirs(wheel_dir)
             command = ["pyproject-build", "-o", wheel_dir]
             if not use_venv:
                 command.append("--no-isolation")
                 env = dict(os.environ)
             else:
+                # Remove pip from environment to let venv install it.
                 env = self.get_no_pip_environment()
             print(f"\nWheel create command: {' '.join(command)}")
-            # Remove pip from environment to let venv install it.
             subprocess.run(command, check=True, cwd=temp_src, env=env)
+            # Remove temporary manually as sometimes git files will cause some
+            # permission error.
+            remove_tree(temp_dir)
         wheel_file_name = [
             name for name in os.listdir(wheel_dir) if name.endswith(".whl")][0]
         return os.path.join(wheel_dir, wheel_file_name)
@@ -619,6 +648,8 @@ class PythonWheelBuilder(PythonBuilder, InstallBuilder):
         """
         wheels = [wheel for wheel in self.get_installers()
                   if wheel.endswith(".whl")]
+        if len(wheels) == 0:
+            raise InstallerNotFoundError("Wheel installer file not found.")
         self.install_wheel(
             # Python installer always only one whl file.
             wheels[0], change_shebang=change_shebang, shebang=shebang)
